@@ -255,7 +255,7 @@ public:
   Integer uninitialized;
 
   Ullman_Array (int array_range, int set_size);
-  void    store (int index, Integer &item);
+  void    store (int index, const Integer &item);
   Integer fetch (int index);
 
 };
@@ -279,7 +279,7 @@ Ullman_Array::Ullman_Array (int array_range, int set_size)
 // are set to point at each other.
 
 inline void
-Ullman_Array::store (int index, Integer &item)
+Ullman_Array::store (int index, const Integer &item)
 {
   int  hand_shake_index = index_array[index];
   
@@ -379,7 +379,7 @@ Memo_Fib::fib3a (int n)
 Integer 
 fib4 (int n)
 {
-#ifdef __GNUC__
+#if defined (__GNUC__) && !defined (__STRICT_ANSI__)
   Integer table[n + 1];
 #else
   Integer *table = new Integer[n + 1];
@@ -411,18 +411,21 @@ struct Extended_Int
   Integer rad_5;
   int     pow_2;
 
-  Extended_Int (Integer &param1, Integer &param2, int param3);
-  Extended_Int (Extended_Int& param);
+  Extended_Int (const Integer &param1, const Integer &param2, int param3);
+  Extended_Int (const Extended_Int& param);
   Extended_Int (void) {}
 
-  friend Extended_Int operator- (Extended_Int &param1, Extended_Int &param2);
-  friend Extended_Int operator* (Extended_Int &param1, Extended_Int &param2);
-  friend Extended_Int sqrt (Extended_Int &param1);
-  friend Extended_Int pow  (Extended_Int &param1, int num);
+  friend inline Extended_Int operator- (const Extended_Int &param1,
+					const Extended_Int &param2);
+  friend Extended_Int operator* (const Extended_Int &param1,
+				 const Extended_Int &param2);
+  friend inline Extended_Int sqrt (const Extended_Int &param1);
+  friend Extended_Int pow  (const Extended_Int &param1, int num);
 };
 
 inline
-Extended_Int::Extended_Int (Integer &param1, Integer &param2, int param3)
+Extended_Int::Extended_Int (const Integer &param1, const Integer &param2,
+			    int param3)
 {
   base  = param1;
   rad_5 = param2;
@@ -430,7 +433,7 @@ Extended_Int::Extended_Int (Integer &param1, Integer &param2, int param3)
 }
 
 inline
-Extended_Int::Extended_Int (Extended_Int &param)
+Extended_Int::Extended_Int (const Extended_Int &param)
 {
   base  = param.base;
   rad_5 = param.rad_5;
@@ -438,7 +441,7 @@ Extended_Int::Extended_Int (Extended_Int &param)
 }
 
 inline Extended_Int 
-operator- (Extended_Int &param1, Extended_Int &param2)
+operator- (const Extended_Int &param1, const Extended_Int &param2)
 {
   Extended_Int temp;
   temp.base  = param1.base - param2.base;
@@ -448,7 +451,7 @@ operator- (Extended_Int &param1, Extended_Int &param2)
 }
 
 Extended_Int 
-operator* (Extended_Int &param1, Extended_Int &param2)
+operator* (const Extended_Int &param1, const Extended_Int &param2)
 {
   Extended_Int temp;
   temp.base  = param1.base * param2.base + 5L * param1.rad_5 * param2.rad_5;
@@ -466,13 +469,13 @@ operator* (Extended_Int &param1, Extended_Int &param2)
 }
 
 inline Extended_Int 
-sqrt (Extended_Int &param1)
+sqrt (const Extended_Int &param1)
 {
   return param1 * param1;
 }
 
 Extended_Int 
-pow (Extended_Int &param1, int num) 
+pow (const Extended_Int &param1, int num) 
 {
   if (num > 1)
     return odd (num)
