@@ -1,8 +1,8 @@
 // Use all the g++ headerfiles
 
-// $Author: law $
-// $Revision: 1.12 $
-// $Date: 1993/11/24 20:42:03 $
+// $Author: bothner $
+// $Revision: 1.15 $
+// $Date: 1994/10/14 23:05:46 $
 
 #include <_G_config.h>
 // If we have the old iostream library, it defines _OLD_STREAMS
@@ -12,17 +12,11 @@
 
 #include <sys/types.h>
 #include <sys/time.h>
-#if _G_HAVE_SYS_RESOURCE
-#include <sys/resource.h>
-#endif
 #include <sys/file.h>
-#include <fcntl.h>
-#include <sys/param.h>
+#if _G_HAVE_SYS_WAIT
 #include <sys/wait.h>
-#include <sys/signal.h>
-#if _G_HAVE_SYS_SOCKET
-#include <sys/socket.h>
 #endif
+#include <sys/signal.h>
 #include <sys/stat.h>
 #include <sys/times.h>
 
@@ -54,7 +48,6 @@
 #include <Integer.h>
 #include <Incremental.h>
 #include <LogNorm.h>
-#include <fcntl.h>
 #include <NegExp.h>
 #include <Normal.h>
 #include <Obstack.h>
@@ -87,7 +80,6 @@
 #include <new.h>
 #include <osfcn.h>
 #include <pwd.h>
-#include <regex.h>
 #include <setjmp.h>
 #include <signal.h>
 #include <stdarg.h>
