@@ -88,7 +88,10 @@ void
 _IO_file_init(fp)
      register _IO_FILE *fp;
 {
-  fp->_offset = _IO_pos_0;
+  /* POSIX.1 allows another file handle to be used to change the position
+     of our file descriptor.  Hence we actually don't know the actual
+     position before we do the first fseek (and until a following fflush). */
+  fp->_offset = _IO_pos_BAD;
   fp->_IO_file_flags |= CLOSED_FILEBUF_FLAGS;
 
   _IO_link_in(fp);
@@ -117,7 +120,7 @@ _IO_file_close_it(fp)
   _IO_un_link(fp);
   fp->_flags = _IO_MAGIC|CLOSED_FILEBUF_FLAGS;
   fp->_fileno = EOF;
-  fp->_offset = _IO_pos_0;
+  fp->_offset = _IO_pos_BAD;
 
   return status;
 }
@@ -211,6 +214,9 @@ _IO_file_setbuf(fp, p, len)
     return 0;
 }
 
+/* Write TO_DO bytes from DATA to FP.
+   Then mark FP has having empty buffers. */
+
 int
 _IO_do_write(fp, data, to_do)
      register _IO_FILE *fp;
@@ -296,18 +302,17 @@ int _IO_file_overflow (f, ch)
 {
   if (f->_flags & _IO_NO_WRITES) /* SET ERROR */
     return EOF;
-  /* If current reading or no buffer allocated. */
+  /* If currently reading or no buffer allocated. */
   if ((f->_flags & _IO_CURRENTLY_PUTTING) == 0)
     {
       /* Allocate a buffer if needed. */
-      if (f->_IO_buf_base == 0)
+      if (f->_IO_write_base == 0)
 	{
 	  _IO_doallocbuf(f);
-	  f->_IO_read_end = f->_IO_buf_base;
-	  f->_IO_write_ptr = f->_IO_buf_base;
+	  _IO_setg (f, f->_IO_buf_base, f->_IO_buf_base, f->_IO_buf_base);
 	}
-      else /* Must be currently reading. */
-	f->_IO_write_ptr = f->_IO_read_ptr;
+      /* Otherwise must be currently reading. */
+      f->_IO_write_ptr = f->_IO_read_ptr;
       f->_IO_write_base = f->_IO_write_ptr;
       f->_IO_write_end = f->_IO_buf_end;
       f->_IO_read_base = f->_IO_read_ptr = f->_IO_read_end;
@@ -347,9 +352,9 @@ _IO_file_sync(fp)
       _IO_off_t new_pos = fp->_jumps->__seek(fp, delta, 1);
       if (new_pos == (_IO_off_t)EOF)
 	return EOF;
-      fp->_offset = new_pos;
       fp->_IO_read_end = fp->_IO_read_ptr;
     }
+  fp->_offset = _IO_pos_BAD;
   /* FIXME: Cleanup - can this be shared? */
   /*    setg(base(), ptr, ptr); */
   return 0;
