@@ -40,7 +40,7 @@ _IO_fprintf
 {
   int ret;
   va_list args;
-  COERCE_FILE(fp);
+  CHECK_FILE(fp, -1);
   _IO_va_start(args, format);
   ret = _IO_vfprintf(fp, format, args);
   va_end(args);
