@@ -23,13 +23,14 @@ This exception does not however invalidate any other reasons why
 the executable file might be covered by the GNU General Public License. */
 
 #include "libioP.h"
+#include <string.h>
 
 int _IO_fputs(str, fp)
      const char *str;
      _IO_FILE *fp;
 {
   _IO_size_t len = strlen(str);
-  COERCE_FILE(fp);
+  CHECK_FILE(fp, EOF);
   if (_IO_sputn(fp, str, len) != len)
     return EOF;
   return 1;
