@@ -32,6 +32,7 @@ the executable file might be covered by the GNU General Public License. */
 #include "libioP.h"
 #include <stdio.h>  /* Needed for sprintf */
 #include <ctype.h>
+#include <string.h>
 #include <limits.h>
 #include "floatio.h"
 
@@ -39,10 +40,10 @@ the executable file might be covered by the GNU General Public License. */
 
 //#define isspace(ch) ((ch)==' ' || (ch)=='\t' || (ch)=='\n')
 
-istream::istream(streambuf *sb, ostream* tied) : ios(sb, tied)
+istream::istream(streambuf *sb, ostream* tied)
 {
-    _flags |= ios::dont_close;
-    _gcount = 0;
+  init (sb, tied);
+  _gcount = 0;
 }
 
 int skip_ws(streambuf* sb)
@@ -68,6 +69,8 @@ istream& istream::get(char& c)
 	  _gcount = 1;
 	}
     }
+    else
+      _gcount = 0;
     return *this;
 }
 
@@ -85,13 +88,13 @@ int istream::peek()
 
 istream& istream::ignore(int n /* = 1 */, int delim /* = EOF */)
 {
+    _gcount = 0;
     if (ipfx1()) {
 	register streambuf* sb = _strbuf;
 	if (delim == EOF) {
 	    _gcount = sb->ignore(n);
 	    return *this;
 	}
-	_gcount = 0;
 	for (;;) {
 #if 0
 	    if (n != MAXINT) // FIXME
@@ -116,8 +119,10 @@ istream& istream::read(char *s, int n)
     if (ipfx1()) {
 	_gcount = _strbuf->sgetn(s, n);
 	if (_gcount != n)
-	    set(ios::failbit);
+	    set(ios::failbit|ios::eofbit);
     }
+    else
+      _gcount = 0;
     return *this;
 }
 
@@ -301,6 +306,16 @@ READ_INT(unsigned long)
 READ_INT(long long)
 READ_INT(unsigned long long)
 #endif
+#if _G_HAVE_BOOL
+READ_INT(bool)
+#endif
+
+istream& istream::operator>>(long double& x)
+{
+    if (ipfx0())
+	scan("%lg", &x);
+    return *this;
+}
 
 istream& istream::operator>>(double& x)
 {
@@ -389,7 +404,7 @@ static void write_int(ostream& stream, unsigned LONGEST val, int sign)
 	    *--buf_ptr = (val & 7) + '0';
 	    val = val >> 3;
 	} while (val != 0);
-	if ((stream.flags() & ios::showbase) && (val != 0))
+	if ((stream.flags() & ios::showbase) && (*buf_ptr != '0'))
 	    *--buf_ptr = '0';
     }
     else if ((stream.flags() & ios::basefield) == ios::hex) { // Hex
@@ -399,7 +414,7 @@ static void write_int(ostream& stream, unsigned LONGEST val, int sign)
 	    *--buf_ptr = xdigs[val & 15];
 	    val = val >> 4;
 	} while (val != 0);
-	if ((stream.flags() & ios::showbase) && (val != 0)) {
+	if ((stream.flags() & ios::showbase)) {
 	    show_base = xdigs + 16; // Either "0X" or "0x".
 	    show_base_len = 2;
 	}
@@ -471,9 +486,10 @@ ostream& ostream::operator<<(int n)
 {
     if (opfx()) {
 	int sign = 1;
+	unsigned int abs_n = (unsigned)n;
 	if (n < 0 && (flags() & (ios::oct|ios::hex)) == 0)
-	    n = -n, sign = -1;
-	write_int(*this, n, sign);
+	    abs_n = -((unsigned)n), sign = -1;
+	write_int(*this, abs_n, sign);
     }
     return *this;
 }
@@ -490,9 +506,9 @@ ostream& ostream::operator<<(long n)
 {
     if (opfx()) {
 	int sign = 1;
-	unsigned int abs_n = (unsigned)n;
+	unsigned long abs_n = (unsigned long)n;
 	if (n < 0 && (flags() & (ios::oct|ios::hex)) == 0)
-	    abs_n = -((unsigned)n), sign = -1;
+	    abs_n = -((unsigned long)n), sign = -1;
 	write_int(*this, abs_n, sign);
     }
     return *this;
@@ -534,9 +550,6 @@ ostream& ostream::operator<<(double n)
 	// stdio implementation.  The setup code uses the same logic
 	// as in __vsbprintf.C (also based on Torek's code).
 	int format_char;
-#if 0
-	if (flags() ios::showpos) sign = '+';
-#endif
 	if ((flags() & ios::floatfield) == ios::fixed)
 	    format_char = 'f';
 	else if ((flags() & ios::floatfield) == ios::scientific)
@@ -544,22 +557,24 @@ ostream& ostream::operator<<(double n)
 	else
 	    format_char = flags() & ios::uppercase ? 'G' : 'g';
 
-	int fpprec = 0; // 'Extra' (suppressed) floating precision.
 	int prec = precision();
+	if (prec <= 0 && !(flags() & ios::fixed))
+	  prec = 6; /* default */
+
+	// Do actual conversion.
+#ifdef _IO_USE_DTOA
+	if (_IO_outfloat(n, rdbuf(), format_char, width(0),
+			 prec, flags(),
+			 flags() & ios::showpos ? '+' : 0,
+			 fill()) < 0)
+	    set(ios::badbit|ios::failbit); // ??
+#else
+	int fpprec = 0; // 'Extra' (suppressed) floating precision.
 	if (prec > MAXFRACT) {
 	    if (flags() & (ios::fixed|ios::scientific) & ios::showpos)
 		fpprec = prec - MAXFRACT;
 	    prec = MAXFRACT;
 	}
-	else if (prec <= 0 && !(flags() & ios::fixed))
-	  prec = 6; /* default */
-
-	// Do actual conversion.
-#ifdef USE_DTOA
-	if (_IO_outfloat(n, rdbuf(), format_char, width(0),
-		       prec, flags(), 0, fill()) < 0)
-	    set(ios::badbit|ios::failbit); // ??
-#else
 	int negative;
 	char buf[BUF];
 	int sign = '\0';
@@ -570,6 +585,7 @@ ostream& ostream::operator<<(double n)
 				&negative,
 				format_char, cp, buf + sizeof(buf));
 	if (negative) sign = '-';
+	else if (flags() & ios::showpos) sign = '+';
 	if (*cp == 0)
 	    cp++;
 
@@ -663,9 +679,9 @@ ostream& ostream::operator<<(register streambuf* sbuf)
   return *this;
 }
 
-ostream::ostream(streambuf* sb, ostream* tied) : ios(sb, tied)
+ostream::ostream(streambuf* sb, ostream* tied)
 {
-    _flags |= ios::dont_close;
+  init (sb, tied);
 }
 
 ostream& ostream::seekp(streampos pos)
@@ -703,7 +719,7 @@ streampos ostream::tellp()
 
 ostream& ostream::flush()
 {
-    if (_strbuf->_jumps->__sync(_strbuf))
+    if (_strbuf->sync())
 	set(ios::badbit);
     return *this;
 }
@@ -772,9 +788,9 @@ void ostream::do_osfx()
     }
 }
 
-iostream::iostream(streambuf* sb, ostream* tied) : ios(sb, tied)
+iostream::iostream(streambuf* sb, ostream* tied)
 {
-  _flags |= ios::dont_close;
+  init (sb, tied);
 }
 
 // NOTE: extension for compatibility with old libg++.
@@ -782,13 +798,10 @@ iostream::iostream(streambuf* sb, ostream* tied) : ios(sb, tied)
 #ifdef _STREAM_COMPAT
 void ios::close()
 {
-  if (!(_flags & (unsigned int)ios::dont_close))
-    delete rdbuf();
-  else if (_strbuf->_flags & _IO_IS_FILEBUF)
+  if (_strbuf->_flags & _IO_IS_FILEBUF)
     ((struct filebuf*)rdbuf())->close();
   else if (_strbuf != NULL)
     rdbuf()->sync();
-  _flags |= ios::dont_close;
   _strbuf = NULL;
   _state = badbit;
 }
