@@ -30,6 +30,6 @@ vfprintf(fp, format, args)
      char const *format;
      _IO_va_list args;
 {
-  COERCE_FILE(fp);
+  CHECK_FILE(fp, -1);
   return _IO_vfprintf(fp, format, args);
 }
